#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ------------
#| Change Log |
# ------------
# Rel 0.1: Initial release. Inspired by plug-ins ofn-bend-path.py https://sourceforge.net/projects/gimp-path-tools/files/scripts/
# Rel 0.2: Add option: "Use Metallic Effects? based by Custom Font Metallic Effects v.0.3 for Gimp 2.10 by Skinnyhouse.
#          https://gimpchat.com/viewtopic.php?f=9&t=17818
# Rel 0.3: Add style: Trumpet. Due to an error in the number of parameters, it was abandoned run interaktive Custom Font Metallic Effects
# Rel 0.4: Add style: Fish, Heart.
# Rel 0.5: add Vertical text
# Rel 0.6: add Drop Shadow.
# Rel 0.7: add "Additional effects(interactive)" - scripts: 3D-Extrusion.scm, Bevel-Reflect-Logo.scm, Chisel.scm, Double_Bump_Map.scm, Gradient Bevel, GT-bevel.scm.
# Rel 0.8: add Ofn-bend-path.py by Ofnuts to this plugin, add Extrude from Path.scm - Created by Graechan

from gimpfu import *

##############################################################################################################################################################################
########################################## Start Ofn-bend-path.py by Ofnuts ##################################################################################################
##############################################################################################################################################################################

import os, sys, math, cmath
from collections import namedtuple
import traceback

debug=False
def trace(s):
    if debug:
        print s

Curve=namedtuple('Curve',['startX','endX','startY','endY','coeffsX','coeffsY' ])

# Functions to determine the position of the reference lines
# All these functions return topY,botY
def referencesFromBestFit(image,path,topY,botY):
    # Due to Y pointing down the bottom is the biggest Y
    topY=min([min(stroke.points[0][1::2]) for stroke in path.strokes])
    botY=max([max(stroke.points[0][1::2]) for stroke in path.strokes])
    return topY,botY
    
def referencesFromExplicitValues(image,path,topY,botY):
    if topY>botY:
        return botY,topY
    return topY,botY

def referencesFromGuides(image,path,topY,botY):
    guide=0
    vPos=[]

    guide=pdb.gimp_image_find_next_guide(image,0)
    while guide!=0: 
        if pdb.gimp_image_get_guide_orientation(image, guide)==ORIENTATION_HORIZONTAL:
            vPos.append(pdb.gimp_image_get_guide_position(image, guide))
        guide=pdb.gimp_image_find_next_guide(image,guide)

    if len(vPos) != 2:
        raise Exception('There should be exactly two horizontal guides')
    return min(vPos),max(vPos)

referencesFunctions=[('Best fit',referencesFromBestFit),('Guides',referencesFromGuides),('Explicit Values',referencesFromExplicitValues)]

# The math

def cardano(d,c,b,a):

    J=cmath.exp(2j*cmath.pi/3)
    Jc=1/J
    z0=b/3/a
    a2,b2 = a*a,b*b
    p=-b2/3/a2 +c/a
    q=(b/27*(2*b2/a2-9*c/a)+d)/a
    D=-4*p*p*p-27*q*q
    r=cmath.sqrt(-D/27+0j)
    u=((-q-r)/2)**0.33333333333333333333333
    v=((-q+r)/2)**0.33333333333333333333333
    w=u*v
    w0=abs(w+p/3)
    w1=abs(w*J+p/3)
    w2=abs(w*Jc+p/3)
    if w0<w1:
        if w2<w0 : v*=Jc
    elif w2<w1 : v*=Jc
    else: v*=J
    return u+v-z0, u*J+v*Jc-z0, u*Jc+v*J-z0

def secondDegree(c,b,a):
    discrim=b*b-4*a*c
    A=(-b/(2*a))+0j # return complex number
    B=math.sqrt(abs(discrim))/(2*a)
    if discrim==0:
        return [A] # there should be only one
    elif discrim>0:
        return [A+B,A-B]
    else: # for completeness since these will be filtered out
        return [A+(B*1j),A-(B*1j)]

def l2r(s):

    if s[0]<s[-2]:
        return s
    else: # reverse the pairs
        return [c for p in reversed(zip(s[0::2],s[1::2])) for c in p]
    
def polynomialCoeffs(p):

    coeffs=[p[0],-3*p[0]+3*p[1],3*p[0]-6*p[1]+3*p[2],-p[0]+3*p[1]-3*p[2]+p[3]]
    return coeffs

def curveFromPoints(points):

    pX=points[0::2]
    pY=points[1::2]
    coeffsX=polynomialCoeffs(pX)
    trace('PointsX: %r, coeffsX: %r' % (pX,coeffsX))
    coeffsY=polynomialCoeffs(pY)
    return Curve(pX[0],pX[-1],pY[0],pY[-1],coeffsX,coeffsY)
    

class Spline(object):

    def __init__(self,stroke):
        self.name='***' # set later, once we know relative positions
        s,_=stroke.points
        s=l2r(s)
        self.curves=[]
        for i in range(2,len(s)-4,6):
            self.curves.append(curveFromPoints(s[i:i+8]))
            
    def dump(self):
        print '------------------ %s' % self.name
        for c in self.curves:
            print '%6.2f -> %6.2f' % (c.startX,c.endX)
            print c.coeffsX
            print c.coeffsY

    def leftY(self):
        return self.curves[0].startY
    
    def rightY(self):
        return self.curves[-1].endY
    
    def xOutsideRange(self,x):
        raise Exception('x=%.2f outside range for "%s" envelope' % (x,self.name))
                        
    def TforX(self,x,curve):

        coeffs=curve.coeffsX[:] # make copy since we alter
        coeffs[0]-=x # "Polynomial(t)=coord" is same as "Polynomial(t)-coord=0"
        d,c,b,a=coeffs
        if a!=0.:
            roots=cardano(d,c,b,a)
        elif b!=0:
            roots=secondDegree(d,c,b)
        elif c!=0:
            roots=[(-d/c)+0j]
        else:
            roots=[d+0j]
            
        #print "Cardano results:", roots
        realRoots=[root.real for root in sorted(roots,key=lambda r: r.real) if 0.<= root.real <= 1. and abs(root.imag) < 1e-8]
        if len(realRoots)==0:
            print 'Roots:',roots
            self.xOutsideRange(x)
        elif len(realRoots)>1:
            print 'Roots:',roots
            raise Exception('Invalid %s envelope shape for x=%.2f' % (self.name,x))
        else:
            return realRoots[0]
    
    def YforT(self,t,curve):

        p=curve.coeffsY
        termPower=1
        y=0
        for n in range(4):
            y+=termPower*p[n]
            termPower*=t
        return y
        
    def YforX(self,x):

        # Find adequate Bezier curve
        for c in self.curves:
            #trace('Checking Curve %.2f -> %.2f for %.2f on %s' % (c.startX,c.endX,x,self.name))
            if c.startX <= x <= c.endX:
                break
        else: # not found 
            self.xOutsideRange(x)

        # The two easy cases. 
        # Making them special cases may avoid round-off nastiness when solving the equations
        if x==c.startX:
            return c.startY
        if x==c.endX:
            return c.endY
        
        t=self.TforX(x,c)
        return self.YforT(t,c)

def coordBetween(c1,c2,r):
    return c1+(c2-c1)*r

def pointBetween(p1,p2,r):
    x1,y1=p1
    x2,y2=p2
    return coordBetween(x1,x2,r),coordBetween(y1,y2,r)

def addBendingPoints(pts,bendiness):

    # To handle the "closing" stroke from last to first without convoluted logic
    # extend each end with opposite end
    pts=pts[-2:]+pts+pts[:2]
    for c in range(0,len(pts)-1,3):
        sa,st,et,ea=pts[c:c+4]
        if sa==st and et==ea: # no tangents
            pts[c+1]=pointBetween(sa,ea,bendiness)
            pts[c+2]=pointBetween(ea,sa,bendiness)
    return pts[2:-2] # drop the ones we added, sight unseen

def bend(points,srcTopY,srcBotY,topSpline,botSpline,bendiness):
    pts=addBendingPoints(zip(points[0::2],points[1::2]),bendiness)
    bentPoints=[]
    # Move the Y corrdinate of each point
    for x,y in pts:
        yRelative=(y-srcTopY)/(srcBotY-srcTopY)
        bendTopY=topSpline.YforX(x)
        bendBotY=botSpline.YforX(x)
        bentY=bendTopY+(bendBotY-bendTopY)*yRelative
        #print 'x=%5.2f, y=%5.2f, yRelative=%5.3f, bendTopY=%5.2f, bendBotY=%5.2f, bentY=%5.2f' % (x,y,yRelative,bendTopY,bendBotY,bentY)
        bentPoints.extend([x,bentY])
    return bentPoints

def order(s1,s2):

    if s1.leftY()<=s2.leftY() and s1.rightY()<=s2.rightY():
        top,bottom=s1,s2 # Smaller Y at top 
    elif s1.leftY()>=s2.leftY() and s1.rightY()>=s2.rightY():
        top,bottom=s2,s1 
    else:
        raise Exception('Ambiguous position of the envelope strokes')
    top.name='top'
    bottom.name='bottom'
    if debug:
        top.dump()
        bottom.dump()
    return top,bottom

def splinesFromEnvelope(envelope):

    if len(envelope.strokes)!=2:
        raise Exception('Envelope path shoud contain exactly two strokes')
    s1=Spline(envelope.strokes[0])
    s2=Spline(envelope.strokes[1])
    return order(s1,s2)
    
def bendPath_text_logo(image,path,envelope,bendiness,references,topY,botY):
    trace('*'*60)
    # Figure out reference lines, and top/bottom envelopes
    srcTopY,srcBotY=referencesFunctions[references][1](image,path,topY,botY)
    bendiness=bendiness*.005 # 100% bendiness is half the distance     
    topSpline,botSpline=splinesFromEnvelope(envelope)
    
    # Bend each stroke
    result=gimp.Vectors(image,'Bent <%s>' % path.name)
    for stroke in path.strokes:
        points,closed=stroke.points
        bentPoints=bend(points,srcTopY,srcBotY,topSpline,botSpline,bendiness)
        gimp.VectorsBezierStroke(result,bentPoints,closed)
    pdb.gimp_image_add_vectors(image,result,0)
    result.visible=True

##############################################################################################################################################################################
########################################## End Ofn-bend-path.py by Ofnuts ####################################################################################################
##############################################################################################################################################################################

def python_fu_text_to_ofn_bend_path(text, vertical, int, font, just, letter_spacing, line_spacing, font_size, background_color, background_opacity, style_top, style_bottom, add_back, fill, fillpattern, other, stroke, flatten, visible_v):

	# set background color
	gimp.set_background(background_color)

	# create image: width, height, type
	image = gimp.Image(font_size, font_size, RGB)

	# create layer with text and add it to the image: image, drawable, x, y, text, border, antialias, size, size_type, fontname
	text_layer = pdb.gimp_text_fontname(image, None, 0, 0, text, 0, 1, font_size, PIXELS, font)

	# justification:
	pdb.gimp_text_layer_set_justification (text_layer, just)
	pdb.gimp_text_layer_set_letter_spacing(text_layer, letter_spacing)
	pdb.gimp_text_layer_set_line_spacing(text_layer, line_spacing)
	
	if vertical==0:	
	# create path in text	
		new_vectors = pdb.gimp_vectors_new(image, text_layer)
		x=pdb.gimp_vectors_new_from_text_layer(image, text_layer)
		
		pdb.gimp_image_insert_vectors(image, x, None, -1)
		active_vectors = pdb.gimp_image_get_active_vectors(image)
		active_vectors.name = active_vectors.name.replace("Text", font)
		active_vectors.visible = False

	if vertical==1:	
		pdb.gimp_text_layer_set_base_direction(text_layer ,5)
		pdb.gimp_image_resize_to_layers(image)	
		new_vectors = pdb.gimp_vectors_new(image, text_layer)
		x=pdb.gimp_vectors_new_from_text_layer(image, text_layer)
		pdb.gimp_image_insert_vectors(image, x, None, -1)
		pdb.gimp_item_transform_rotate_simple(x,0, True, 1,1)
		pdb.gimp_item_transform_rotate_simple(text_layer,0, True, 1,1)		
	
	# resize image to layer content
	pdb.gimp_image_resize_to_layers(image)
	    
	# create layer: image, width, height, type, name, opacity, mode
	layer = gimp.Layer(image, "background", image.width, image.height, RGBA_IMAGE, background_opacity, 0)
	
	# append layer to the image
	image.add_layer(layer, 0)
	
	# create path envelope	
	pdb.gimp_selection_all(image)
	selection = pdb.gimp_selection_bounds(image)
	
	x1 = selection[1];
	y1 = selection[2];
	x2 = selection[3];
	y2 = selection[4];
	
	W1 = 2*(x1+(x2 - x1)/2)
	H1 = 2*(y1+(y2 - y1)/2)
	
	W2 =(x1+(x2 - x1)/2)
	H2 =(y1+(y2 - y1)/2)
	
	W3 = W1/3
	H3 = H1/3
	
	W4 = W2/2
	H4 = H2/2
	
	r=int
	new_vectors_e = pdb.gimp_vectors_new(image,"Envelope")
	pdb.gimp_image_insert_vectors(image, new_vectors_e, None, -1)
	
	pdb.gimp_item_set_visible(new_vectors_e, True)	
# style_top, style_bottom,
	if style_top==0:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y1, x1,y1, x1,y1,  W4,y1-r, W2,-r, W2+W4,y1-r,  W1-x1,y1, W1-x1,y1 ,W1-x1,y1, ), 0)
	if style_top==1:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y1-r, x1,y1-r, x1,y1-r,   W4,y1, W2,y1, W2+W4,y1,   x2,y1-r, x2,y1-r, x2,y1-r), 0)	
	if style_top==2:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y1, x1,y1, x1,y1,   W1-x1,y1, W1-x1,y1 ,W1-x1,y1, ), 0)	
	if style_top==3:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y1, x1,y1, x1,y1,  W2,-r, W2,-r, W2,-r,  W1-x1,y1, W1-x1,y1 ,W1-x1,y1, ), 0)
	if style_top==4:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y1-r, x1,y1-r, x1,y1-r,   W2,y1, W2,y1, W2,y1,   x2,y1-r, x2,y1-r, x2,y1-r), 0)	
	if style_top==5:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y1, x1,y1, x1,y1,  x2,y1-r, x2,y1-r, x2,y1-r), 0)
	if style_top==6:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y1-r, x1,y1-r, x1,y1-r,  W1-x1,y1, W1-x1,y1 ,W1-x1,y1 ), 0)
	if style_top==7:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y1, x1,y1, x1,y1,    W4-r,y1-r,W4,y1-r,W4+r,y1-r,     W2,y1, W2,y1, W2,y1,    x2-W4-r,y1+r, x2-W4,y1+r, x2-W4+r,y1+r,     W1-x1,y1, W1-x1,y1 ,W1-x1,y1, ), 0)	
	if style_top==8:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y1, x1,y1, x1,y1,    W4-r,y1+r,W4,y1+r,W4+r,y1+r,     W2,y1, W2,y1, W2,y1,    x2-W4-r,y1-r,x2-W4,y1-r,x2-W4+r,y1-r,     W1-x1,y1, W1-x1,y1 ,W1-x1,y1, ), 0)	
	if style_top==9:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y1, x1,y1, x1,y1,  W3,y1-r, W3,-r, W3,y1-r,   2*W3,y1-r, 2*W3,-r, 2*W3,y1-r,   W1-x1,y1, W1-x1,y1 ,W1-x1,y1, ), 0)
	if style_top==10:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y1, x1,y1, x1,y1,  W3,y1+r, W3,r, W3,y1+r,    2*W3,y1+r, 2*W3,r, 2*W3,y1+r,   W1-x1,y1, W1-x1,y1 ,W1-x1,y1, ), 0)
	if style_top==11:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y1, x1,y1, x1,y1,   x2,y1, W1-x1,y1-r ,W1-x1,y1-r, ), 0)
	if style_top==12:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y1-r, x1,y1-r, x1,y1,   W1-x1,y1, W1-x1,y1 ,W1-x1,y1, ), 0)
	if style_top==13:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y1-r, x1,y1-r, x1,y1-r,  W4,y1+r/2, W4,y1+r/2, W4,y1+r/2,   2*W3-2*r,y1-r, 2*W3,y1-r, 2*W3+2*r,y1-r,  W1-x1,y1+r/2, W1-x1,y1+r/2 ,W1-x1,y1+r/2), 0)
	if style_top==14:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y1+r/2, x1,y1+r/2 ,x1,y1+r/2,    W3-2*r,y1-r, W3,y1-r, W3+2*r,y1-r,   3*W4,y1+r/2, 3*W4,y1+r/2, 3*W4,y1+r/2,   x2,y1-r, x2,y1-r, x2,y1-r,), 0)
	if style_top==15:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y1+r/3, x1,y1+r/3, x1,y1-r/2,    W4-r/2,y1-r, W4,y1-r, W4+r/2,y1-r,   W2,y1-r/3, W2,y1+r/4, W2,y1-r/3,    3*W4-r/2,y1-r, 3*W4,y1-r, 3*W4+r/2,y1-r,   W1-x1,y1-r/2, W1-x1,y1+r/3 ,W1-x1,y1+r/3), 0)		
	
	
	
	if style_bottom==0:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y2+r, x1,y2+r, x1,y2+r,   W4,y2, W2,y2, W2+W4,y2,   x2,y2+r, x2,y2+r, x2,y2+r), 0)		
	if style_bottom==1:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y2, x1,y2, x1,y2,   W4,y2+r, W2,y2+r, W2+W4,y2+r,   x2,y2, x2,y2, x2,y2), 0)	
	if style_bottom==2:   # line
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y2, x1,y2, x1,y2,   W1-x1,y2, W1-x1,y2 ,W1-x1,y2, ), 0)		
	if style_bottom==3: 
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y2+r, x1,y2+r, x1,y2+r,   W2,y2, W2,y2, W2,y2,   x2,y2+r, x2,y2+r, x2,y2+r), 0)	
	if style_bottom==4:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y2, x1,y2, x1,y2,   W2,y2+r, W2,y2+r, W2,y2+r,   x2,y2, x2,y2, x2,y2), 0)	
	if style_bottom==5:	
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y2+r, x1,y2+r, x1,y2+r,    x2,y2, x2,y2, x2,y2), 0)
	if style_bottom==6:	
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y2, x1,y2, x1,y2,  x2,y2+r, x2,y2+r, x2,y2+r), 0)		
	if style_bottom==7:	
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y2, x1,y2, x1,y2,  W4-r,y2-r,W4,y2-r,W4+r,y2-r,  W2,y2, W2,y2, W2,y2,  x2-W4-r,y2+r, x2-W4,y2+r, x2-W4+r,y2+r,     W1-x1,y2, W1-x1,y2 ,W1-x1,y2, ), 0)
	if style_bottom==8:	
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 30, (x1,y2, x1,y2, x1,y2,    W4-r,y2+r,W4,y2+r,W4+r,y2+r,     W2,y2, W2,y2, W2,y2,    x2-W4-r,y2-r,x2-W4,y2-r,x2-W4+r,y2-r,     W1-x1,y2, W1-x1,y2 ,W1-x1,y2, ), 0)
	if style_bottom==9:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y2+r, x1,y2+r, x1,y2+r,   W3,y2, W3,y2, W3,y2,    2*W3,y2, 2*W3,y2, 2*W3,y2,   W1-x1,y2+r, W1-x1,y2+r ,W1-x1,y2+r, ), 0)	
	if style_bottom==10: 
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y2, x1,y2, x1,y2,    W3,y2+r, W3,y2+r,W3,y2+r,   2*W3,y2+r,2*W3,y2+r,2*W3,y2+r,   W1-x1,y2, W1-x1,y2 ,W1-x1,y2, ), 0)
	if style_bottom==11: 
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y2, x1,y2, x1,y2,   W1-x1,y2, W1-x1,y2+r ,W1-x1,y2+r, ), 0)	
	if style_bottom==12:  
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 12, (x1,y2+r, x1,y2+r, x1,y2,  W1-x1,y2, W1-x1,y2 ,W1-x1,y2,  ), 0)	
	if style_bottom==13:	
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y2+r, x1,y2+r, x1,y2+r,   W4,y2-r/2, W4,y2-r/2, W4,y2-r/2,    2*W3-2*r,y2+r, 2*W3,y2+r, 2*W3+2*r,y2+r,   W1-x1,y2-r/2, W1-x1,y2-r/2 ,W1-x1,y2-r/2, ), 0)		
	if style_bottom==14:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 24, (x1,y2-r/2, x1,y2-r/2 ,x1,y2-r/2,    W3-2*r,y2+r, W3,y2+r, W3+2*r,y2+r,   3*W4,y2-r/2, 3*W4,y2-r/2, 3*W4,y2-r/2,   x2,y2+r, x2,y2+r, x2,y2+r,), 0)
	if style_bottom==15:
		pdb.gimp_vectors_stroke_new_from_points(new_vectors_e, 0, 18, (x1,y2-r/4, x1,y2-r/4, x1,y2-r/4,   W2,y2+3*r/2, W2,y2+3*r/2, W2,y2+3*r/2,   x2,y2-r/4, x2,y2-r/4, x2,y2-r/4), 0)	
	
	b=add_back+r/2
	pdb.gimp_layer_resize(layer, x2-x1+2*b, y2-y1+2*int+2*b,0+b,int+b,)
	pdb.gimp_image_resize_to_layers(image)	
	
	try:
		gimp.Display(image)
	except:
		pass

	pdb.gimp_image_set_active_vectors(image, x)	
# bendPath_text_logo(image,path,envelope,bendiness,references,topY,botY)
	bendPath_text_logo(image ,x ,new_vectors_e , int, 0, 0, 0)
	active_vectors = pdb.gimp_image_get_active_vectors(image)
	x.visible = False
	pdb.gimp_image_remove_layer(image, text_layer)
	pdb.gimp_selection_none(image)
	layer.fill(1)
	
	pdb.gimp_selection_none(image)
	new_vectors = []
	if vertical==1:	
		pdb.gimp_item_transform_rotate_simple(active_vectors,2, True, 1,1)
		pdb.gimp_item_transform_rotate_simple(layer,2, True, 1,1)
		pdb.gimp_image_resize_to_layers(image)
		#pdb.gimp_image_remove_vectors(image, new_vectors_e)
		#pdb.gimp_image_remove_vectors(image, x)
# FILL-FOREGROUND (0), FILL-BACKGROUND (1), FILL-WHITE (2), FILL-TRANSPARENT (3), FILL-PATTERN (4)
	if fill==1:   # "With Pattern", 
		W = pdb.gimp_image_width(image)  #szer
		H = pdb.gimp_image_height(image)
	
	#Select the vectors, this is like front end's "Path To Selection"
		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,active_vectors)
		fill_layer = gimp.Layer(image, "Fill Pattern", W, H, RGBA_IMAGE, 100, 0)
		image.add_layer(fill_layer, -1)
		
	#Set context pattern and fill selection with pattern fill
		pdb.gimp_context_set_pattern(fillpattern)
     
		pdb.gimp_edit_fill(fill_layer, 4)	  # 4 = FILL-PATTERN 

	# Now we select None so that Stroke path call later will stroke the whole path and not just selected area
		pdb.gimp_selection_none(image)

	if fill==2:	#  "With Foreground", 
		W = pdb.gimp_image_width(image)
		H = pdb.gimp_image_height(image)

		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,active_vectors)
		fill_layer = gimp.Layer(image, "Fill Foreground", W, H, RGBA_IMAGE, 100, 0)
		image.add_layer(fill_layer, -1)

		pdb.gimp_edit_fill(fill_layer, 0)     #  0= FILL-FOREGROUND

		pdb.gimp_selection_none(image)

	if fill==3:   # "With Background"
		W = pdb.gimp_image_width(image)
		H = pdb.gimp_image_height(image)

		pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,active_vectors)
		fill_layer = gimp.Layer(image, "Fill Background", W, H, RGBA_IMAGE, 100, 0)
		image.add_layer(fill_layer, -1)
     
		pdb.gimp_edit_fill(fill_layer, 1)	   # 1 = FILL-BACKGROUND

		pdb.gimp_selection_none(image)
	if other>0:		
	#if other==1 or other==2 or other==3 or other==4 or other==5 or other==6 or other==7 or other==8:
		if fill==0:
			W = pdb.gimp_image_width(image) 
			H = pdb.gimp_image_height(image)
			pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,active_vectors)
			fill_layer = gimp.Layer(image, "Fill WHITE", W, H, RGBA_IMAGE, 100, 0)
			image.add_layer(fill_layer, -1)
			pdb.gimp_context_set_pattern(fillpattern)
			pdb.gimp_edit_fill(fill_layer, 2)	
	
		temp_copy_org = pdb.gimp_layer_copy(fill_layer, True)    # add MrQ
		image.add_layer(temp_copy_org, -1)			
	if other==1:
		pdb.python_layer_fx_drop_shadow(image, temp_copy_org, (0,0,0), 100, 1, 1, 0, 1, 10, 90, 1, False, False,run_mode=RUN_INTERACTIVE)
	if other==2:		
		pdb.script_fu_3d_extrude(image ,temp_copy_org ,0 ,10 ,135 ,125 ,45 ,1 ,0.3 ,0 ,False ,False ,False ,False ,"Dried mud",run_mode=RUN_INTERACTIVE)
	if other==3:			
		pdb.script_fu_3d_outline_logo_alpha(image ,temp_copy_org , "Abstract 1" ,0 ,0 ,0 ,0 ,0 ,0,run_mode=RUN_INTERACTIVE)
	if other==4:
		pdb.script_fu_ml_bevel_reflect_logo_alpha(image ,temp_copy_org ,0 ,0, 8 ,10 ,7 ,20 ,1 ,0 ,0, 1,"none" ,0,0,0,0 ,"Abstract 1" ,315 ,0.35 ,0 ,1.2 ,2.6 ,300 ,0 ,True ,True ,True,run_mode=RUN_INTERACTIVE)
	if other==5:	
		pdb.script_fu_chisel(image ,temp_copy_org ,20 ,5 ,0 ,0 ,135 ,30 ,20 ,0 ,0 ,0 ,False,run_mode=RUN_INTERACTIVE)
	if other==6:
		pdb.script_fu_double_bump_map(image ,temp_copy_org ,(0, 129, 229) ,"Pattern", 2 ,3 ,135 ,35 ,35 ,4 ,4 ,15 ,(0, 0, 0) ,60, run_mode=RUN_INTERACTIVE)
	if other==7:
		pdb.gimp_image_set_active_vectors(image, active_vectors)
		pdb.script_fu_extrude_from_path(image, temp_copy_org, 20, 45, 8, 15, 80, "1. Pixel", 3, 0, "Blue", "Shadows 1", False, 0, "Red", False, True, run_mode=RUN_INTERACTIVE)		
	if other==8:
		pdb.script_fu_gradient_bevel_logo_alpha(image,temp_copy_org, 22,40,2.5, (255,255,255),run_mode=RUN_INTERACTIVE)
	if other==9:
		pdb.script_fu_gt_bevel(image ,temp_copy_org, 1,10,15,135,45,0.5,False,False,True,run_mode=RUN_INTERACTIVE)
	
	if stroke==1:
		pdb.gimp_selection_none(image)
		W = pdb.gimp_image_width(image)
		H = pdb.gimp_image_height(image)
		stroke_b = gimp.Layer(image, "Stroke brush", W, H, RGBA_IMAGE, 100, 0)
		image.add_layer(stroke_b, -1)
		pdb.gimp_edit_stroke_vectors(stroke_b ,active_vectors)
		
	if stroke==2 or stroke==3:
		if fill==0:
			W = pdb.gimp_image_width(image)  #szer
			H = pdb.gimp_image_height(image)

			pdb.gimp_image_select_item(image,CHANNEL_OP_REPLACE,active_vectors)
			fill_layer = gimp.Layer(image, "Fill", W, H, RGBA_IMAGE, 100, 0)
			image.add_layer(fill_layer, -1)

			#pdb.gimp_context_set_pattern(fillpattern)

		 
		if stroke==2:
			pdb.python_layer_fx_inner_glow(image ,fill_layer, (256,256,256) ,100 ,1 ,1 ,1 ,1 ,1 ,10 ,FALSE,run_mode=RUN_INTERACTIVE)


		if stroke==3:
			pdb.python_layer_fx_outer_glow(image ,fill_layer, (256,256,256) ,100 ,1 ,1 ,1, 1, 1 ,FALSE ,FALSE, run_mode=RUN_INTERACTIVE)
	#pdb.gimp_item_set_visible(layer, 0)	

	pdb.gimp_selection_none(image)
	if not other==7:
		pdb.gimp_image_remove_vectors(image, x)
		pdb.gimp_image_remove_vectors(image, new_vectors_e)

	if flatten:
	    image.merge_visible_layers(CLIP_TO_IMAGE)
		
		
	if visible_v==0:
		pdb.gimp_item_set_visible(active_vectors, False)
	
	
#=====================================================================================================================			

register(
	"python_fu_text_to_ofn-bend-path",
	"Creating bend-text-logo v.0.8. Necessary for Interactive Action: \n- Non-interactive: Ofn-bend-path.py by Ofnuts\n- : LayerFX.py, scripts: 3D-Extrusion, 3D Outline, Bevel-Reflect-Logo, Chisel, Double-Bump-Map, Extrude From Path ,Gradient Bevel, GT-Bevel.",
	"Creates logo with specified text and ofn-bend-path.py",
	"GPL",
	"MareroQ",
	"2019",
	"Bend Text-Logo...",
	"",
	[
        (PF_TEXT, 	"text", ("Text:"), "GIMPCHAT"),
        (PF_BOOL,  "vertical", "Vertical text?", False),		
        (PF_SPINNER, "int", "Intensity (max. bending distance):", 50, (0, 1000, 1)),
        (PF_FONT, "font", "Font:", "Arial"),
        (PF_OPTION, "just", "Justify:", 2, ["LEFT","RIGHT","CENTER"]),
        (PF_SPINNER, "letter_spacing", "Letter Spacing:", 0, (-100, 100, 1, 5, 0, 0)),
        (PF_SPINNER, "line_spacing", "Line Spacing:", 0, ( -100, 100, 1, 5, 0, 0)),
        (PF_INT, "font_size", "Font size:", 150),
        (PF_COLOR, "background_color", "Background color:", (255, 255, 255)),
        (PF_SLIDER, "background_opacity", "Background opacity:", 100, (0, 100, 1)),
        (PF_OPTION,   "style_top", "Top envelope style:", 0, [
        u'\u25E0'"    Bulging up",
        u'\u25E1'"    Bulging down",
        u'\uFE58'"    A straight line",		
        u'\u2227'"    Bent up (roof)",
        u'\u2228'"    Bent down (wedge)",		
        u'\uFF0F'"    Slash",		
        u'\uFF3C'"    BackSlash",
        u'\u223C'"    Wave (tilde)",		
        u'\u223D'"    Wave (reversed tilde)",
        u'\uFE39'"    Tortoise shell",
        u'\uFE3A'"    Tortoise shell reversed",	
        u'\u25DE'"    Arc Trumpet right (upper)",
        u'\u25DF'"    Arc Trumpet left (upper)",
        u'\u21A3'"    Fish right (upper)",
        u'\u21A2'"    Fish left (upper)",
        u'\u25E0\u25E0'"   Heart (upper)",		
        ]),
        (PF_OPTION,   "style_top", "Bottom envelope style:", 1, [
        u'\u25E0'"    Bulging up",
        u'\u25E1'"    Bulging down",
        u'\uFE58'"    A straight line",
        u'\u2227'"    Bent up (roof)",
        u'\u2228'"    Bent down (wedge)",
        u'\uFF0F'"    Slash",		
        u'\uFF3C'"    BackSlash",
        u'\u223C'"    Wave (tilde)",		
        u'\u223D'"    Wave (reversed tilde)",
        u'\uFE39'"    Tortoise shell",
        u'\uFE3A'"    Tortoise shell reversed",
        u'\u25DD'"    Arc Trumpet right (lower)",
        u'\u25DC'"    Arc Trumpet left (lower)",
        u'\u21A3'"    Fish right (lower)",
        u'\u21A2'"    Fish left (lower)",
        " V     Heart (lower)",		
        ]),
        (PF_SPINNER, "add_back", "Enlarge BG (add margin-px):", 10, (0, 1000, 1, 5, 0, 0)),		
        (PF_OPTION, "fill", "FILL (path to selection)?:", 0, ["Don't fill", "With Pattern", "With Foreground", "With Background"]),		
        (PF_PATTERN, "fillpattern", "Fill Pattern:", 0),
        (PF_OPTION, "other", "Additional effects:\n (interactive)", 0, ["Don't use", "FX Drop Shadow", "3D Extrusion", "3D Outline","Bevel Reflect Logo","Chisel", "Double Bump Map","Extrude From Path","Gradient Bevel","GT Bevel"]),		
        (PF_OPTION, "stroke", "Contour:", 0, ["Don't use", "With active brush and FG color", "FX Inner Glow", "FX Outer Glow"]),
        (PF_BOOL,  "flatten", "Merge layers?", False),
        (PF_BOOL,  "visible_v", "Path Text visible ?", False),		
	],
	[],
	python_fu_text_to_ofn_bend_path,
	menu="<Image>/File/Create/Logos",
	)

main()

